library(WDI)
library(dplyr)
library(ggplot2)
library(sf)
library(rnaturalearth)
library(rnaturalearthdata)
library(RColorBrewer)

# Define the file path for the cached population data
cache_file <- "population_data.csv"

# Function to fetch and cache population data
fetch_population_data <- function() {
  # Fetch population data for the year 2022
  population_data <- WDI(country = "all", indicator = "SP.POP.TOTL", start = 2022, end = 2022)
  population_data <- population_data %>%
    select(iso2c, country, SP.POP.TOTL) %>%
    rename(population = SP.POP.TOTL)
  
  # Save the fetched data to a CSV file
  write.csv(population_data, file = cache_file, row.names = FALSE)
  return(population_data)
}

# Check if the cached file exists
if (file.exists(cache_file)) {
  # Read the cached population data
  population_data <- read.csv(cache_file, stringsAsFactors = FALSE)
} else {
  # Fetch and cache the population data
  population_data <- fetch_population_data()
}

# Read the Developers' Data from the CSV File
developers_data <- read.csv("dd-by-country.csv", stringsAsFactors = FALSE)

# Ensure column names match expected names
colnames(developers_data) <- c("number_of_developers", "country_code")

# Merge the Data
merged_data <- developers_data %>%
  left_join(population_data, by = c("country_code" = "iso2c"))

# Check for missing population data and print a warning if necessary
missing_population <- merged_data %>% filter(is.na(population))
if (nrow(missing_population) > 0) {
  warning("The following countries were excluded due to missing population data:\n",
          paste(missing_population$country_code, collapse = ", "))
}

# Remove rows with NA values in the population column
merged_data <- merged_data %>% filter(!is.na(population))

# Calculate the Incidence of Debian Developers per 100,000 Inhabitants
merged_data <- merged_data %>%
  mutate(incidence = (number_of_developers / population) * 100000)

# Save the Result to CSV
write.csv(merged_data, file = "debian_developers_incidence.csv", row.names = FALSE)

# Load the world map data
world <- ne_countries(scale = "medium", returnclass = "sf")

# Merge the incidence data with the world map data
world <- left_join(world, merged_data, by = c("iso_a2_eh" = "country_code"))

# Create a color palette
color_palette <- colorRampPalette(c("blue", "red"))(1000)

# Plot the world map with the incidence data
ggplot(data = world) +
  geom_sf(aes(fill = incidence), color = "black", size = 0.25) +
  scale_fill_gradientn(colors = color_palette, na.value = "white", name = "Incidence") +
  labs(title = "Incidence of Debian Developers per 100,000 Inhabitants",
       caption = "Countries with no data are shown in white") +
  theme_minimal() +
  theme(
    legend.position = "right",
    panel.grid = element_blank(),
    axis.text = element_blank(),
    axis.ticks = element_blank(),
    axis.title = element_blank(),
    plot.background = element_rect(fill = "transparent", color = NA),  # Transparent plot background
    panel.background = element_rect(fill = "transparent", color = NA), # Transparent panel background
    legend.background = element_rect(fill = "transparent", color = NA) # Transparent legend background
  )

# Save the Plot as a PDF with transparent background
ggsave("debian_developers_incidence_world_map.pdf", width = 14, height = 7, bg = "transparent")

# Print a message indicating completion
print("Incidence calculation and world map plotting completed. Results saved to debian_developers_incidence.csv and debian_developers_incidence_world_map.pdf")

